% Script to process multiple cylinder displacement-time histories
% Input: Multiple cylinder displacement-time history .csv files
% respectively
clc
clear all
close all

%% Common parameters
numCylinders = 2; % #cylinders used in analysis of interference
D = 0.0889; %diameter of cylinders [m]
rhoFluid = 999.1026; %density of fluid [kg/m^3]
kinematicViscosity = 0.0000011389; %[m^2/s]
%L/D between the corresponding cylinder index and the previous cylinder
horizantalSpacingOverD = [0, 1.50]; %enter the dimensionless horizontal spacing vector
mOsc = [7.923 7.4]; %enter the oscillating mass of each cylinders in [kg]
dampingStructural = [9.264, 3.060]; %structural damping coefficient vector according to fNat,air [Nm/s]
VCKstatus = [1 1]; %every column defines whether the corresponding cylinder
%is a VCK (1) or a real-spring system(2). Enter the value starting from the
%1st cylinder followed by downstream direction cylinders
motorFlowSpeedCoeff = 0.02008;
%% Analysis/Plot Parameters
Options.Plots.plotResponseStatus = 1; %1: Plot individual time-history response of each cylinder, 
                                  %0: No individual response plot
Options.TickProcess.VCKsampleRate = 0.005; %VCK time sampling rate in [s]
Options.TickProcess.realSpringSampleRate = 0.01; %Real-spring system time sampling rate in [s]
Options.TickProcess.minThresholdAmplitude = 0.01; %min. peak amplitude to process
Options.TickProcess.numPeaksToCount = 30; %number of peak points to process
% max seconds to allow no peak behavior to appear in oscillations 
Options.TickProcess.peakDetectTimeThreshold = 10; %[s] 
%% Prepare data structures and the data to be read                                 
cylinder = struct('spacing', {horizantalSpacingOverD}); %initialize cylinder struct
for ID = 1:numCylinders
  % Prepare the files to be read
  cylinder(ID).spacing =  horizantalSpacingOverD(ID); %assign spacings
  [cylinder(ID).file.fileName, cylinder(ID).file.pathName] = ... 
   uigetfile({'*.csv';'*.dat'}, ...
              ['Select Cylinder #' num2str(ID) ' displacement-time history data file(s)'], ...
              'MultiSelect', 'on');
  
  fName = cylinder(ID).file.fileName;         
  if isequal(fName, 0)
    error('User selected Cancel') %when no data is selected
  else %when user selects tick data files to be processed

  % convert filenames to a cell array of strings
  if ~iscellstr(fName), fName = cellstr(fName); end     
    disp(['User selected the following file(s) for Cylinder #' num2str(ID) ':']);
    fprintf('%s\n', fName{:});
  end
  %% Obtain the average A* amplitude, A* time-history, standard dev. of the motion
  % signal, frequency of oscillation, and the time-vector for the cylinders  
  %IMPORTANT! This part needs to be optimized for a unified function call
  % Read the ticks for Real spring or VCK cylinder motion amplitude from fileID 

  numFlowSpeed = numel(fName); % # of experiment flow speeds
                                                    % to analyze data from
  %initilaze a column-vector of A* for each flow speed experiment analyzed
  cylinder(ID).Astar = zeros(numFlowSpeed,1); 
  %initialize a column-vector of Frequency of Osc. for each experiment flow-speed analyzed
  cylinder(ID).frequencyOsc = zeros(numFlowSpeed,1); %[Hz]
  %initialize a column-vector of standard dev. of motion response for each experiment flow speed analyzed
  cylinder(ID).STD = zeros(numFlowSpeed,1);
  %initilaze a cell array of displacement-history for each flow speed experiment analyzed
  cylinder(ID).Astar = zeros(numFlowSpeed,1); 
  %initialize a column-vector of motor frequencies at which experiments were run 
  cylinder(ID).motorFrequencies = zeros(numFlowSpeed,1); %[Hz]
  %initialize a column-vector of Re# at which experiments were run 
  cylinder(ID).reynolds = zeros(numFlowSpeed,1);
  %initialize a column-vector of U* in air at which experiments were run 
  cylinder(ID).UstarAir = zeros(numFlowSpeed,1);
  %assign the status of cylinder as 1 if it's VCK, 0 if it is Real-spring
  cylinder(ID).isVCK = VCKstatus(ID);%decides if the system is VCK or not
  %find and assign mStar for each cylinder
  cylinder(ID).mStar = mOsc(ID)/(rhoFluid*pi/4*D^2);
  %% Grab test parameters from the filename
  if ( cylinder(ID).isVCK) %when the cylinder is attached to a VCK system
    %grab stiffness term from the experiment data file  
    tempIndex = strfind(fName{1}, '-k'); %finds where "k" term is in fName
  	temp = strsplit(fName{1}(tempIndex+2:end), '-');
    cylinder(ID).K = str2double(temp{1}); %spring stiffness of the cylinder
    %grab damping ratio term from the experiment data file 
     tempIndex = strfind(fName{1}, '-z'); %finds where "zeta" term is in fName
  	temp = strsplit(fName{1}(tempIndex+2:end), '-');
    cylinder(ID).zeta = str2double(temp{1}); %damping ratio of the cylinder
  else %when the cylinder is attached to a real-spring system
    tempIndex = strfind(fName{1}, 'K_'); %finds where "k" term is in fName
  	temp = strsplit(fName{1}(tempIndex+2:end), '_');
    cylinder(ID).K = str2double(temp{1}); %spring stiffness of the cylinder
  end
  % fName = strrep(fName, '.csv', '');
  
  %% Compute cylinder vibration parameters from stiffness and zeta
  cylinder(ID).naturalFrequencyAir = (1/(2*pi)* ... 
                                     sqrt((cylinder(ID).K)/mOsc(ID)));

  %% Process the data files for each flow speed experiment carried out
  fprintf('\nCylinderID\t A/D\t STD\t Osc. Freq.\t Motor Freq.\t UstarAir\t Re#\n');
 for iFlowSpeed = 1:numFlowSpeed %counter to loop over multiple experiment flow-speeds 

    cylinder(ID).file.fileID = fopen([cylinder(ID).file.pathName ... 
                                      fName{iFlowSpeed}]);
    singleFlowSpeedTestData = textscan(cylinder(ID).file.fileID, ...
                              '%f %f %f %f %f', 'delimiter', ',');
    fclose(cylinder(ID).file.fileID); %close the file

    if (cylinder(ID).isVCK)  %when the cylinder is a VCK system
       % read experiment motor frequency from filename
       tempIndex = strfind(fName{iFlowSpeed}, '-f'); %finds where freq. is
       %-f is 2 characters and the motor frequency is assumed to be in
       %double digits
       cylinder(ID).motorFrequencies(iFlowSpeed) = ...
                   str2num(fName{iFlowSpeed}(tempIndex+2:tempIndex+3));
       %process VCK system cylinder data file   
      [cylinder(ID).Astar(iFlowSpeed), cylinder(ID).motionHistory, ...
       cylinder(ID).STD(iFlowSpeed), cylinder(ID).frequencyOsc(iFlowSpeed), ...
       cylinder(ID).recordedTime, mintab, maxtab] = ...
       computeVCKAstarAndTimeHistory(singleFlowSpeedTestData, Options.TickProcess);

    else %when the cylinder is a Real spring-system
      % read experiment motor frequency from filename
      tempIndex = strfind(fName{iFlowSpeed}, 'Hz_');
      %Motor frequency in Hz. 'Hz_' is 3 characters and the motor frequency
      %is assumed to be in double digits
      cylinder(ID).motorFrequencies(iFlowSpeed) = ...
                   str2num(fName{iFlowSpeed}(tempIndex+3:tempIndex+4));
      %process real spring-system cylinder data file          
      [cylinder(ID).Astar(iFlowSpeed), cylinder(ID).motionHistory, ...
       cylinder(ID).STD(iFlowSpeed), cylinder(ID).frequencyOsc(iFlowSpeed), ...
       cylinder(ID).recordedTime, mintab, maxtab] = ...
      computeRealAstarAndTimeHistory(singleFlowSpeedTestData, Options.TickProcess);
    end
    %compute the absolute flow speed [m/s] from motor frequency
    flowSpeed = motorFlowSpeedCoeff * cylinder(ID).motorFrequencies(iFlowSpeed);
    flowSpeed = flowSpeed - 0.0054; %parametrize the magic number here!!!
    cylinder(ID).reynolds(iFlowSpeed) = flowSpeed * D / kinematicViscosity;
    cylinder(ID).UstarAir(iFlowSpeed) = flowSpeed / ...
                                        (cylinder(ID).naturalFrequencyAir * D); 
    %% Print results
    fprintf('\t%s \t %.4f \t %.3f \t %.3f \t\t %s \t\t %2.3f \t %.4g\n', ...
            num2str(ID), cylinder(ID).Astar(iFlowSpeed), ...
            cylinder(ID).STD(iFlowSpeed), ...
            cylinder(ID).frequencyOsc(iFlowSpeed), ...
            num2str(cylinder(ID).motorFrequencies(iFlowSpeed)), ...
            cylinder(ID).UstarAir(iFlowSpeed), ...
            cylinder(ID).reynolds(iFlowSpeed));
    
    %% Plots
    % Plot individual motion time history of a cylinder and mark its max-min responses
    if Options.Plots.plotResponseStatus == 1
      figure; plot(cylinder(ID).recordedTime,cylinder(ID).motionHistory);
      hold on; plot(mintab(:,1), mintab(:,2), 'g*');
      plot(maxtab(:,1), maxtab(:,2), 'r*');
      xlabel('Time [s]')
      ylabel('Dimensionless Amplitude, A/D')
      title( ['Displacement time-history of Cylinder #' num2str(ID) ...
              ' @ Motor Frequency = ' ...
              num2str(cylinder(ID).motorFrequencies(iFlowSpeed)) ...
              ', UstarAir = ' ...
              num2str(cylinder(ID).UstarAir(iFlowSpeed)) ...
              ', Re# = ' ...
              num2str(cylinder(ID).reynolds(iFlowSpeed))] )
      hold off;
    end
    
% Plot motion time histories of the cylinders on top of each other
%     figure;
%     plot(cylinder(1).recordedTime, cylinder(1).motionHistory, 'b', 'LineWidth', 1)
%     grid on
%     hold on
%     plot(cylinder(2).recordedTime, cylinder(2).motionHistory, 'r', 'LineWidth', 1)
%     xlim([0 60])
%     ylim([-1.1*max(max(cylinder.Astar)) 1.1*max(max(cylinder.Astar))])
%     set(gca, 'FontName', 'Times New Roman', 'FontSize', 14)
%     xlabel('Time [s]')
%     ylabel('Amplitude, A/D')
%     title('Two cylinder motion time-history recordings')
%     legend('First signal', 'Second signal')
%     hold off
 end
end
close all